## begin license ##
#
#    Slowfoot is an integration system for web-based applications.
#    Copyright (C) 2005-2009 Seek You Too B.V. (CQ2) http://www.cq2.nl
#
#    This file is part of Slowfoot.
#
#    Slowfoot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Slowfoot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Slowfoot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##

import os
from md5 import md5
from os.path import join, abspath, dirname, isfile

class AccountException(Exception):
	pass

class Account:
	def __init__(self, username):
		self._username = username

	def username(self):
		return self._username


def _encryptPasswd(aString):
    return md5(aString).hexdigest()

def _validUsername(aString):
    return aString.strip() != '' and not ':' in aString

def _validPassword(aString):
    return aString.strip() != ''

def createAccount(username, password, passwordFile):
    if not _validUsername(username) or not _validPassword(password):
        raise AccountException("Invalid username/password")
    
    users = _readPasswordFile(passwordFile)
    users.append((username, _encryptPasswd(password)))
    _writePasswordFile(users, passwordFile)
    return Account(username)
    
def deleteAccount(aUsername, passwordFile):
    users = dict(_readPasswordFile(passwordFile))
    if aUsername in users:
        del users[aUsername]
    _writePasswordFile(users.items(), passwordFile)
    
def validUserUID(aString):
    return False

def exists(aUsername, passwordFile):
    for username, password in _readPasswordFile(passwordFile):
        if aUsername == username:
            return True
    return  False

def login(aUsername, aPassword, passwordFile):
    encryptedPassword = _encryptPasswd(aPassword)
    for username, password in _readPasswordFile(passwordFile):
        if aUsername == username and encryptedPassword == password:
            return Account(username)

    raise AccountException('Invalid username/password')

def _readPasswordFile(aFilename):
    if aFilename == None or not isfile(aFilename):
        raise IOError('Password file does not exist.')
    result = []
    lines = open(aFilename).readlines()
        
    for line in lines:
        result.append(line.strip().split(':'))
    return result

def _writePasswordFile(userList, aFilename):
    fd = open(aFilename, 'w')
    try:
        for username, password in userList:
            print >> fd, '%s:%s' % (username, password)
    finally:
        fd.close()
    