## begin license ##
#
#    Slowfoot is an integration system for web-based applications.
#    Copyright (C) 2005-2009 Seek You Too B.V. (CQ2) http://www.cq2.nl
#
#    This file is part of Slowfoot.
#
#    Slowfoot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Slowfoot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Slowfoot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##

from mod_python import apache, psp, util, Session
from slowfoot.request import Request
import tempfile, StringIO
from lrucache import LRUCache
from threading import Lock
import sys,os

assert sys.getdefaultencoding() == 'utf-8'

stats = {}
pspcache = LRUCache(64)
lock = Lock()
slowfootplugins = apache.import_module('slowfoot.slowfootplugins')
activeplugins = slowfootplugins.preopenplugins
	
def prof(req):
	args = req.args and util.parse_qs(req.args) or {}
	sort = args.get('sort', ['time'])[0]
	strstream = StringIO.StringIO()
	sys.stdout = strstream
	stats['data'].sort_stats(sort)
	stats['data'].print_stats()
	stats['data'].print_callers()
	stats['data'].print_callees()
	req.write("<pre>" + strstream.getvalue() + "</pre>")
	return apache.OK

def handler(req):
	req = Request(req, handlepsp, util, _psp_login, retrieveSession)
	
	for plugin in activeplugins:
		req.addPlugin('PREOPEN', plugin)
	req.go()
	return apache.OK

#To enable profile:
# - rename profilinghandler to handler (disable other handler)
# - uncomment the following import:
# import profile, pstats
def profilinghandler(req):
	if req.uri == '/profile': return prof(req)
	profiledata = tempfile.NamedTemporaryFile()
	try:
		global _profiling_req
		_profiling_req = req
		profile.run(
		"""
from request import Request
import slowfoothandler
from mod_python import util
Request(slowfoothandler._profiling_req, slowfoothandler.handlepsp, util, 
		slowfoothandler._psp_login).go()""",
			profiledata.name)
		if not stats.get('data', ''):
			stats['data'] = pstats.Stats(profiledata.name)
		else:
			stats['data'].add(profiledata.name)
		_profiling_req = None
	finally:
		profiledata.close()
	return apache.OK

def retrieveSession(req):
	s = Session.Session(req, lock = 0)
	return s
		
def handlepsp(target, templatename, data):
	psp.PSP(target, filename = templatename).run(data)

def _psp_login():
	raise apache.SERVER_RETURN, apache.HTTP_UNAUTHORIZED
