## begin license ##
#
#    Slowfoot is an integration system for web-based applications.
#    Copyright (C) 2005-2009 Seek You Too B.V. (CQ2) http://www.cq2.nl
#
#    This file is part of Slowfoot.
#
#    Slowfoot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Slowfoot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Slowfoot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
## end license ##
import os, time
import glob
from os.path import isdir, dirname

class SlowfootStorage:

    def __init__(self, documentRoot, templatesPath=None):
        if not isdir(documentRoot):
            raise SystemError('documentRoot should be a directory')
        self._datadir = documentRoot
        self._templatesdir = documentRoot
        if templatesPath != None and isdir(templatesPath):
            self._templatesdir = templatesPath

    def getModifiedDate(self, filename, fullPathGiven=False):
        try:
            fullPath = fullPathGiven and filename or self.getFullPath(filename)
            return os.stat(fullPath).st_mtime
        except OSError:
            return time.time() + 5

    def glob(self, pattern='*'):
        return [os.path.basename(f) for f in glob.glob(self._datadir+'/'+pattern)]


    def _fixNameForBackwardsCompatibility(self, name, mode):
        if 'w' in mode and name.startswith('/data/'):
            name = name[len('/data/'):]
        if dirname(name) not in [self._datadir, self._templatesdir] and name.startswith('/'):
            name = name[1:]
        return name

    def getFullPath(self, name, mode = 'r'):
        name = self._fixNameForBackwardsCompatibility(name,mode)
        if mode != 'r':
            return os.path.join(self._datadir, name)
        for adir in [ self._datadir, self._templatesdir]:
            if os.path.isfile(os.path.join(adir, name)):
                return os.path.join(adir, name)
        return os.path.join(self._datadir, name)

    def opendatastream(self, name, mode = 'r'):
        return open(self.getFullPath(name, mode), mode)
